@@ -1,7 +1,7 @@
 /*
    Various utilities
 
-   Copyright (C) 1994-2015
+   Copyright (C) 1994-2021
    Free Software Foundation, Inc.
 
    Written by:
@@ -40,7 +40,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -52,7 +51,6 @@
 #include "lib/vfs/vfs.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
-#include "lib/timer.h"
 
 /*** global variables ****************************************************************************/
 
@@ -76,14 +74,17 @@
 
 /*** file scope variables ************************************************************************/
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
+#ifndef HAVE_CHARSET
 static inline int
 is_7bit_printable (unsigned char c)
 {
     return (c > 31 && c < 127);
 }
+#endif
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -121,19 +122,19 @@ resolve_symlinks (const vfs_path_t * vpath)
     r = buf = g_malloc (MC_MAXPATHLEN);
     buf2 = g_malloc (MC_MAXPATHLEN);
     *r++ = PATH_SEP;
-    *r = 0;
+    *r = '\0';
 
     do
     {
         q = strchr (p + 1, PATH_SEP);
-        if (!q)
+        if (q == NULL)
         {
-            q = strchr (p + 1, 0);
+            q = strchr (p + 1, '\0');
             if (q == p + 1)
                 break;
         }
         c = *q;
-        *q = 0;
+        *q = '\0';
         if (mc_lstat (vpath, &mybuf) < 0)
         {
             MC_PTR_FREE (buf);
@@ -151,14 +152,14 @@ resolve_symlinks (const vfs_path_t * vpath)
                 MC_PTR_FREE (buf);
                 goto ret;
             }
-            buf2[len] = 0;
+            buf2[len] = '\0';
             if (IS_PATH_SEP (*buf2))
                 strcpy (buf, buf2);
             else
                 strcpy (r, buf2);
         }
         canonicalize_pathname (buf);
-        r = strchr (buf, 0);
+        r = strchr (buf, '\0');
         if (*r == '\0' || !IS_PATH_SEP (r[-1]))
             /* FIXME: this condition is always true because r points to the EOL */
         {
@@ -203,6 +204,7 @@ mc_util_write_backup_content (const char *from_file_name, const char *to_file_na
 
     if (fwrite ((const void *) contents, 1, length, backup_fd) != length)
         ret1 = FALSE;
+
     {
         int ret2;
 
@@ -212,6 +214,7 @@ mc_util_write_backup_content (const char *from_file_name, const char *to_file_na
         ret2 = fclose (backup_fd);
         (void) ret2;
     }
+
     g_free (contents);
     return ret1;
 }
@@ -234,11 +237,9 @@ is_printable (int c)
         return is_7bit_printable (c);
 
     if (mc_global.full_eight_bits)
-    {
         return is_8bit_printable (c);
-    }
-    else
-        return is_iso_printable (c);
+
+    return is_iso_printable (c);
 #endif /* !HAVE_CHARSET */
 }
 
@@ -296,6 +297,8 @@ name_quote (const char *s, gboolean quote_percent)
             if (ret->len == 0)
                 g_string_append_c (ret, '\\');
             break;
+        default:
+            break;
         }
         g_string_append_c (ret, *s);
     }
@@ -328,7 +331,7 @@ path_trunc (const char *path, size_t trunc_len)
 
     vpath = vfs_path_from_str (path);
     secure_path = vfs_path_to_str_flags (vpath, 0, VPF_STRIP_PASSWORD);
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 
     ret = str_trunc (secure_path, trunc_len);
     g_free (secure_path);
@@ -343,26 +346,26 @@ size_trunc (uintmax_t size, gboolean use_si)
 {
     static char x[BUF_TINY];
     uintmax_t divisor = 1;
-    const char *xtra = "";
+    const char *xtra = _("B");
 
     if (size > 999999999UL)
     {
         divisor = use_si ? 1000 : 1024;
-        xtra = use_si ? "k" : "K";
+        xtra = use_si ? _("kB") : _("KiB");
 
         if (size / divisor > 999999999UL)
         {
             divisor = use_si ? (1000 * 1000) : (1024 * 1024);
-            xtra = use_si ? "m" : "M";
+            xtra = use_si ? _("MB") : _("MiB");
 
             if (size / divisor > 999999999UL)
             {
                 divisor = use_si ? (1000 * 1000 * 1000) : (1024 * 1024 * 1024);
-                xtra = use_si ? "g" : "G";
+                xtra = use_si ? _("GB") : _("GiB");
             }
         }
     }
-    g_snprintf (x, sizeof (x), "%.0f%s", 1.0 * size / divisor, xtra);
+    g_snprintf (x, sizeof (x), "%.0f %s", 1.0 * size / divisor, xtra);
     return x;
 }
 
@@ -380,7 +383,9 @@ size_trunc_sep (uintmax_t size, gboolean use_si)
     p += strlen (p) - 1;
     d = x + sizeof (x) - 1;
     *d-- = '\0';
-    while (p >= y && isalpha ((unsigned char) *p))
+    /* @size format is "size unit", i.e. "[digits][space][letters]".
+       Copy all charactes after digits. */
+    while (p >= y && !g_ascii_isdigit (*p))
         *d-- = *p--;
     for (count = 0; p >= y; count++)
     {
@@ -573,14 +578,18 @@ string_perm (mode_t mode_bits)
 const char *
 extension (const char *filename)
 {
-    const char *d = strrchr (filename, '.');
-    return (d != NULL) ? d + 1 : "";
+    const char *d;
+
+    d = strrchr (filename, '.');
+
+    return d != NULL ? d + 1 : "";
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-load_mc_home_file (const char *from, const char *filename, char **allocated_filename)
+load_mc_home_file (const char *from, const char *filename, char **allocated_filename,
+                   size_t * length)
 {
     char *hintfile_base, *hintfile;
     char *lang;
@@ -590,18 +599,18 @@ load_mc_home_file (const char *from, const char *filename, char **allocated_file
     lang = guess_message_value ();
 
     hintfile = g_strconcat (hintfile_base, ".", lang, (char *) NULL);
-    if (!g_file_get_contents (hintfile, &data, NULL, NULL))
+    if (!g_file_get_contents (hintfile, &data, length, NULL))
     {
         /* Fall back to the two-letter language code */
         if (lang[0] != '\0' && lang[1] != '\0')
             lang[2] = '\0';
         g_free (hintfile);
         hintfile = g_strconcat (hintfile_base, ".", lang, (char *) NULL);
-        if (!g_file_get_contents (hintfile, &data, NULL, NULL))
+        if (!g_file_get_contents (hintfile, &data, length, NULL))
         {
             g_free (hintfile);
             hintfile = hintfile_base;
-            g_file_get_contents (hintfile_base, &data, NULL, NULL);
+            g_file_get_contents (hintfile_base, &data, length, NULL);
         }
     }
 
@@ -626,9 +635,9 @@ extract_line (const char *s, const char *top)
     static char tmp_line[BUF_MEDIUM];
     char *t = tmp_line;
 
-    while (*s && *s != '\n' && (size_t) (t - tmp_line) < sizeof (tmp_line) - 1 && s < top)
+    while (*s != '\0' && *s != '\n' && (size_t) (t - tmp_line) < sizeof (tmp_line) - 1 && s < top)
         *t++ = *s++;
-    *t = 0;
+    *t = '\0';
     return tmp_line;
 }
 
@@ -654,7 +663,7 @@ x_basename (const char *s)
     {
         /* avoid trailing PATH_SEP, if present */
         if (!IS_PATH_SEP (s[strlen (s) - 1]))
-            return (path_sep != NULL) ? path_sep + 1 : s;
+            return path_sep + 1;
 
         while (--path_sep > s && !IS_PATH_SEP (*path_sep))
             ;
@@ -666,7 +675,7 @@ x_basename (const char *s)
     while (--url_delim > s && !IS_PATH_SEP (*url_delim))
         ;
 
-    return (url_delim == s) ? s : url_delim + 1;
+    return url_delim == s ? s : url_delim + 1;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -691,8 +700,8 @@ skip_separators (const char *s)
 {
     const char *su = s;
 
-    for (; *su; str_cnext_char (&su))
-        if (*su != ' ' && *su != '\t' && *su != ',')
+    for (; *su != '\0'; str_cnext_char (&su))
+        if (!whitespace (*su) && *su != ',')
             break;
 
     return su;
@@ -705,7 +714,7 @@ skip_numbers (const char *s)
 {
     const char *su = s;
 
-    for (; *su; str_cnext_char (&su))
+    for (; *su != '\0'; str_cnext_char (&su))
         if (!str_isdigit (su))
             break;
 
@@ -756,9 +765,9 @@ strip_ctrl_codes (char *s)
                  * OSC P s ; P t ST
                  * OSC P s ; P t BEL
                  */
-                char *new_r = r;
+                char *new_r;
 
-                for (; *new_r != '\0'; ++new_r)
+                for (new_r = r; *new_r != '\0'; new_r++)
                 {
                     switch (*new_r)
                     {
@@ -768,11 +777,14 @@ strip_ctrl_codes (char *s)
                         goto osc_out;
                     case ESC_CHAR:
                         /* ST */
-                        if (*(new_r + 1) == '\\')
+                        if (new_r[1] == '\\')
                         {
                             r = new_r + 1;
                             goto osc_out;
                         }
+                        break;
+                    default:
+                        break;
                     }
                 }
               osc_out:
@@ -818,9 +830,7 @@ get_compression_type (int fd, const char *name)
 
     /* GZIP_MAGIC and OLD_GZIP_MAGIC */
     if (magic[0] == 037 && (magic[1] == 0213 || magic[1] == 0236))
-    {
         return COMPRESSION_GZIP;
-    }
 
     /* PKZIP_MAGIC */
     if (magic[0] == 0120 && magic[1] == 0113 && magic[2] == 003 && magic[3] == 004)
@@ -840,30 +850,36 @@ get_compression_type (int fd, const char *name)
 
     /* PACK_MAGIC and LZH_MAGIC and compress magic */
     if (magic[0] == 037 && (magic[1] == 036 || magic[1] == 0240 || magic[1] == 0235))
-    {
         /* Compatible with gzip */
         return COMPRESSION_GZIP;
-    }
 
     /* BZIP and BZIP2 files */
     if ((magic[0] == 'B') && (magic[1] == 'Z') && (magic[3] >= '1') && (magic[3] <= '9'))
-    {
         switch (magic[2])
         {
         case '0':
             return COMPRESSION_BZIP;
         case 'h':
             return COMPRESSION_BZIP2;
+        default:
+            break;
         }
-    }
 
-    /* Support for LZMA (only utils format with magic in header).
-     * This is the default format of LZMA utils 4.32.1 and later. */
+    /* LZ4 format - v1.5.0 - 0x184D2204 (little endian) */
+    if (magic[0] == 0x04 && magic[1] == 0x22 && magic[2] == 0x4d && magic[3] == 0x18)
+        return COMPRESSION_LZ4;
 
     if (mc_read (fd, (char *) magic + 4, 2) != 2)
         return COMPRESSION_NONE;
 
-    /* LZMA utils format */
+    /* LZIP files */
+    if (magic[0] == 'L'
+        && magic[1] == 'Z'
+        && magic[2] == 'I' && magic[3] == 'P' && (magic[4] == 0x00 || magic[4] == 0x01))
+        return COMPRESSION_LZIP;
+
+    /* Support for LZMA (only utils format with magic in header).
+     * This is the default format of LZMA utils 4.32.1 and later. */
     if (magic[0] == 0xFF
         && magic[1] == 'L'
         && magic[2] == 'Z' && magic[3] == 'M' && magic[4] == 'A' && magic[5] == 0x00)
@@ -875,6 +891,9 @@ get_compression_type (int fd, const char *name)
         && magic[2] == 0x7A && magic[3] == 0x58 && magic[4] == 0x5A && magic[5] == 0x00)
         return COMPRESSION_XZ;
 
+    if (magic[0] == 0x28 && magic[1] == 0xB5 && magic[2] == 0x2F && magic[3] == 0xFD)
+        return COMPRESSION_ZSTD;
+
     str_len = strlen (name);
     /* HACK: we must belive to extension of LZMA file :) ... */
     if ((str_len > 5 && strcmp (&name[str_len - 5], ".lzma") == 0) ||
@@ -897,10 +916,18 @@ decompress_extension (int type)
         return "/ubz" VFS_PATH_URL_DELIMITER;
     case COMPRESSION_BZIP2:
         return "/ubz2" VFS_PATH_URL_DELIMITER;
+    case COMPRESSION_LZIP:
+        return "/ulz" VFS_PATH_URL_DELIMITER;
+    case COMPRESSION_LZ4:
+        return "/ulz4" VFS_PATH_URL_DELIMITER;
     case COMPRESSION_LZMA:
         return "/ulzma" VFS_PATH_URL_DELIMITER;
     case COMPRESSION_XZ:
         return "/uxz" VFS_PATH_URL_DELIMITER;
+    case COMPRESSION_ZSTD:
+        return "/uzst" VFS_PATH_URL_DELIMITER;
+    default:
+        break;
     }
     /* Should never reach this place */
     fprintf (stderr, "Fatal: decompress_extension called with an unknown argument\n");
@@ -912,13 +939,14 @@ decompress_extension (int type)
 void
 wipe_password (char *passwd)
 {
-    char *p = passwd;
+    if (passwd != NULL)
+    {
+        char *p;
 
-    if (!p)
-        return;
-    for (; *p; p++)
-        *p = 0;
-    g_free (passwd);
+        for (p = passwd; *p != '\0'; p++)
+            *p = '\0';
+        g_free (passwd);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -933,45 +961,49 @@ wipe_password (char *passwd)
 char *
 convert_controls (const char *p)
 {
-    char *valcopy = g_strdup (p);
+    char *valcopy;
     char *q;
 
+    valcopy = g_strdup (p);
+
     /* Parse the escape special character */
-    for (q = valcopy; *p;)
-    {
-        if (*p == '\\')
+    for (q = valcopy; *p != '\0';)
+        switch (*p)
         {
+        case '\\':
             p++;
-            if ((*p == 'e') || (*p == 'E'))
+
+            if (*p == 'e' || *p == 'E')
             {
                 p++;
                 *q++ = ESC_CHAR;
             }
-        }
-        else
-        {
+            break;
+
+        case '^':
+            p++;
             if (*p == '^')
-            {
-                p++;
-                if (*p == '^')
-                    *q++ = *p++;
-                else
-                {
-                    char c = (*p | 0x20);
-                    if (c >= 'a' && c <= 'z')
-                    {
-                        *q++ = c - 'a' + 1;
-                        p++;
-                    }
-                    else if (*p)
-                        p++;
-                }
-            }
-            else
                 *q++ = *p++;
+            else
+            {
+                char c;
+
+                c = *p | 0x20;
+                if (c >= 'a' && c <= 'z')
+                {
+                    *q++ = c - 'a' + 1;
+                    p++;
+                }
+                else if (*p != '\0')
+                    p++;
+            }
+            break;
+
+        default:
+            *q++ = *p++;
         }
-    }
-    *q = 0;
+
+    *q = '\0';
     return valcopy;
 }
 
@@ -984,9 +1016,9 @@ convert_controls (const char *p)
 char *
 diff_two_paths (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 {
-    char *p, *q, *r, *s, *buf = NULL;
-    int i, j, prevlen = -1, currlen;
+    int j, prevlen = -1, currlen;
     char *my_first = NULL, *my_second = NULL;
+    char *buf = NULL;
 
     my_first = resolve_symlinks (vpath1);
     if (my_first == NULL)
@@ -998,25 +1030,27 @@ diff_two_paths (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 
     for (j = 0; j < 2; j++)
     {
+        char *p, *q;
+        int i;
+
         p = my_first;
         q = my_second;
+
         while (TRUE)
         {
-            r = strchr (p, PATH_SEP);
-            s = strchr (q, PATH_SEP);
-            if (r == NULL || s == NULL)
-                break;
-            *r = '\0';
-            *s = '\0';
-            if (strcmp (p, q) != 0)
-            {
-                *r = PATH_SEP;
-                *s = PATH_SEP;
-                break;
-            }
+            char *r, *s;
+            ptrdiff_t len;
 
-            *r = PATH_SEP;
-            *s = PATH_SEP;
+            r = strchr (p, PATH_SEP);
+            if (r == NULL)
+                break;
+            s = strchr (q, PATH_SEP);
+            if (s == NULL)
+                break;
+
+            len = r - p;
+            if (len != (s - q) || strncmp (p, q, (size_t) len) != 0)
+                break;
 
             p = r + 1;
             q = s + 1;
@@ -1194,7 +1228,7 @@ save_file_position (const vfs_path_t * filename_vpath, long line, long column, o
     gboolean src_error = FALSE;
 
     if (filepos_max_saved_entries == 0)
-        filepos_max_saved_entries = mc_config_get_int (mc_main_config, CONFIG_APP_SECTION,
+        filepos_max_saved_entries = mc_config_get_int (mc_global.main_config, CONFIG_APP_SECTION,
                                                        "filepos_max_saved_entries", 1024);
 
     fn = mc_config_get_full_path (MC_FILEPOS_FILE);
@@ -1266,9 +1300,8 @@ extern int
 ascii_alpha_to_cntrl (int ch)
 {
     if ((ch >= ASCII_A && ch <= ASCII_Z) || (ch >= ASCII_a && ch <= ASCII_z))
-    {
         ch &= 0x1f;
-    }
+
     return ch;
 }
 
@@ -1281,7 +1314,8 @@ Q_ (const char *s)
 
     result = _(s);
     sep = strchr (result, '|');
-    return (sep != NULL) ? sep + 1 : result;
+
+    return sep != NULL ? sep + 1 : result;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1292,16 +1326,15 @@ mc_util_make_backup_if_possible (const char *file_name, const char *backup_suffi
     struct stat stat_buf;
     char *backup_path;
     gboolean ret;
+
     if (!exist_file (file_name))
         return FALSE;
 
     backup_path = g_strdup_printf ("%s%s", file_name, backup_suffix);
-
     if (backup_path == NULL)
         return FALSE;
 
     ret = mc_util_write_backup_content (file_name, backup_path);
-
     if (ret)
     {
         /* Backup file will have same ownership with main file. */
@@ -1351,7 +1384,7 @@ mc_util_unlink_backup_if_possible (const char *file_name, const char *backup_suf
 
         vpath = vfs_path_from_str (backup_path);
         mc_unlink (vpath);
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
     }
 
     g_free (backup_path);
@@ -1379,15 +1412,14 @@ guess_message_value (void)
         NULL
     };
 
-    unsigned i = 0;
+    size_t i;
     const char *locale = NULL;
 
-    while (var[i] != NULL)
+    for (i = 0; var[i] != NULL; i++)
     {
         locale = getenv (var[i]);
         if (locale != NULL && locale[0] != '\0')
             break;
-        i++;
     }
 
     if (locale == NULL)
@@ -1396,6 +1428,30 @@ guess_message_value (void)
     return g_strdup (locale);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+/**
+ * The "profile root" is the tree under which all of MC's user data &
+ * settings are stored.
+ *
+ * It defaults to the user's home dir. The user may override this default
+ * with the environment variable $MC_PROFILE_ROOT.
+ */
+const char *
+mc_get_profile_root (void)
+{
+    static const char *profile_root = NULL;
+
+    if (profile_root == NULL)
+    {
+        profile_root = g_getenv ("MC_PROFILE_ROOT");
+        if (profile_root == NULL || *profile_root == '\0')
+            profile_root = mc_config_get_home_dir ();
+    }
+
+    return profile_root;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /**
  * Propagate error in simple way.
@@ -1457,16 +1513,16 @@ mc_replace_error (GError ** dest, int code, const char *format, ...)
  * and if it has then updates the timestamp.
  *
  * @param timestamp the last timestamp in microseconds, updated if the given time elapsed
- * @param deleay amount of time in microseconds
+ * @param delay amount of time in microseconds
 
  * @return TRUE if clock skew detected, FALSE otherwise
  */
 gboolean
-mc_time_elapsed (guint64 * timestamp, guint64 delay)
+mc_time_elapsed (gint64 * timestamp, gint64 delay)
 {
-    guint64 now;
+    gint64 now;
 
-    now = mc_timer_elapsed (mc_global.timer);
+    now = g_get_real_time ();
 
     if (now >= *timestamp && now < *timestamp + delay)
         return FALSE;
