@@ -37,6 +37,7 @@
 #define VFS_SUBCLASS(a) ((struct vfs_s_subclass *) (a))
 
 #define VFS_SUPER(a) ((struct vfs_s_super *) (a))
+#define CONST_VFS_SUPER(a) ((const struct vfs_s_super *) (a))
 #define VFS_ENTRY(a) ((struct vfs_s_entry *) (a))
 #define VFS_INODE(a) ((struct vfs_s_inode *) (a))
 
@@ -87,11 +88,11 @@ struct vfs_s_inode
     struct vfs_s_entry *ent;    /* Our entry in the parent directory -
                                    use only for directories because they
                                    cannot be hardlinked */
-    GList *subdir;              /* If this is a directory, its entry. List of vfs_s_entry */
+    GQueue *subdir;             /* If this is a directory, its entry. List of vfs_s_entry */
     struct stat st;             /* Parameters of this inode */
     char *linkname;             /* Symlink's contents */
     char *localname;            /* Filename of local file, if we have one */
-    struct timeval timestamp;   /* Subclass specific */
+    gint64 timestamp;           /* Subclass specific */
     off_t data_offset;          /* Subclass specific */
 };
 
@@ -139,7 +140,7 @@ struct vfs_s_subclass
                                        struct vfs_s_inode * root,
                                        const char *path, int follow, int flags);
     int (*dir_load) (struct vfs_class * me, struct vfs_s_inode * ino, char *path);
-    int (*dir_uptodate) (struct vfs_class * me, struct vfs_s_inode * ino);
+    gboolean (*dir_uptodate) (struct vfs_class * me, struct vfs_s_inode * ino);
     int (*file_store) (struct vfs_class * me, vfs_file_handler_t * fh, char *path, char *localname);
 
     int (*linear_start) (struct vfs_class * me, vfs_file_handler_t * fh, off_t from);
