@@ -1,7 +1,7 @@
 /*
    Virtual File System: GNU Tar file system.
 
-   Copyright (C) 2000-2019
+   Copyright (C) 2000-2021
    Free Software Foundation, Inc.
 
    Written by:
@@ -217,8 +217,8 @@ cpio_free_archive (struct vfs_class *me, struct vfs_s_super *super)
         mc_close (arch->fd);
         arch->fd = -1;
     }
-    g_slist_free_full (arch->deferred, g_free);
-    arch->deferred = NULL;
+
+    g_clear_slist (&arch->deferred, g_free);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -254,7 +254,7 @@ cpio_open_cpio_file (struct vfs_class *me, struct vfs_s_super *super, const vfs_
         s = g_strconcat (super->name, decompress_extension (type), (char *) NULL);
         tmp_vpath = vfs_path_from_str_flags (s, VPF_NO_CANON);
         fd = mc_open (tmp_vpath, O_RDONLY);
-        vfs_path_free (tmp_vpath);
+        vfs_path_free (tmp_vpath, TRUE);
         if (fd == -1)
         {
             message (D_ERROR, MSG_ERROR, _("Cannot open cpio archive\n%s"), s);
@@ -603,11 +603,12 @@ cpio_read_bin_head (struct vfs_class *me, struct vfs_s_super *super)
 #ifdef HAVE_STRUCT_STAT_ST_RDEV
     st.st_rdev = u.buf.c_rdev;
 #endif
-    st.st_size = (u.buf.c_filesizes[0] << 16) | u.buf.c_filesizes[1];
+    st.st_size = ((off_t) u.buf.c_filesizes[0] << 16) | u.buf.c_filesizes[1];
 #ifdef HAVE_STRUCT_STAT_ST_MTIM
     st.st_atim.tv_nsec = st.st_mtim.tv_nsec = st.st_ctim.tv_nsec = 0;
 #endif
-    st.st_atime = st.st_mtime = st.st_ctime = (u.buf.c_mtimes[0] << 16) | u.buf.c_mtimes[1];
+    st.st_atime = st.st_mtime = st.st_ctime =
+        ((time_t) u.buf.c_mtimes[0] << 16) | u.buf.c_mtimes[1];
 
     return cpio_create_entry (me, super, &st, name);
 }
@@ -890,7 +891,7 @@ void
 vfs_init_cpiofs (void)
 {
     /* FIXME: cpiofs used own temp files */
-    vfs_init_subclass (&cpio_subclass, "cpiofs", VFS_READONLY, "ucpio");
+    vfs_init_subclass (&cpio_subclass, "cpiofs", VFSF_READONLY, "ucpio");
     vfs_cpiofs_ops->read = cpio_read;
     vfs_cpiofs_ops->setctl = NULL;
     cpio_subclass.archive_check = cpio_super_check;
