@@ -7,7 +7,7 @@
    Parts of this program were taken from the lsdel.c and dump.c files
    written by Ted Ts'o (tytso@mit.edu) for the ext2fs package.
 
-   Copyright (C) 1995-2015
+   Copyright (C) 1995-2021
    Free Software Foundation, Inc.
 
    Written by:
@@ -48,16 +48,8 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <fcntl.h>
-
-#ifdef HAVE_EXT2FS_EXT2_FS_H
+#include <string.h>             /* memset() */
 #include <ext2fs/ext2_fs.h>
-#else
-/* asm/types.h defines its own umode_t */
-#undef umode_t
-#include <linux/ext2_fs.h>
-#endif
-
 #include <ext2fs/ext2fs.h>
 #include <ctype.h>
 
@@ -65,6 +57,7 @@
 
 #include "lib/util.h"
 #include "lib/widget.h"         /* message() */
+#include "lib/vfs/xdirentry.h"
 #include "lib/vfs/utilvfs.h"
 #include "lib/vfs/vfs.h"
 
@@ -130,8 +123,11 @@ static char *block_buf;
 static const char *undelfserr = N_("undelfs: error");
 static int readdir_ptr;
 static int undelfs_usage;
-static struct vfs_class vfs_undelfs_ops;
 
+static struct vfs_s_subclass undelfs_subclass;
+static struct vfs_class *vfs_undelfs_ops = VFS_CLASS (&undelfs_subclass);
+
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -241,13 +237,13 @@ undelfs_loaddel (void)
     delarray = g_try_malloc (sizeof (struct deleted_info) * max_delarray);
     if (!delarray)
     {
-        message (D_ERROR, undelfserr, _("not enough memory"));
+        message (D_ERROR, undelfserr, "%s", _("not enough memory"));
         return 0;
     }
     block_buf = g_try_malloc (fs->blocksize * 3);
     if (!block_buf)
     {
-        message (D_ERROR, undelfserr, _("while allocating block buffer"));
+        message (D_ERROR, undelfserr, "%s", _("while allocating block buffer"));
         goto free_delarray;
     }
     retval = ext2fs_open_inode_scan (fs, 0, &scan);
@@ -293,7 +289,8 @@ undelfs_loaddel (void)
                                                                    (max_delarray + 50));
                 if (!delarray_new)
                 {
-                    message (D_ERROR, undelfserr, _("no more memory while reallocating array"));
+                    message (D_ERROR, undelfserr, "%s",
+                             _("no more memory while reallocating array"));
                     goto error_out;
                 }
                 delarray = delarray_new;
@@ -369,13 +366,13 @@ undelfs_opendir (const vfs_path_t * vpath)
         message (D_ERROR, undelfserr, _("Cannot open file %s"), ext2_fname);
         return 0;
     }
-    vfs_print_message (_("undelfs: reading inode bitmap..."));
+    vfs_print_message ("%s", _("undelfs: reading inode bitmap..."));
     if (ext2fs_read_inode_bitmap (fs))
     {
         message (D_ERROR, undelfserr, _("Cannot load inode bitmap from:\n%s"), ext2_fname);
         goto quit_opendir;
     }
-    vfs_print_message (_("undelfs: reading block bitmap..."));
+    vfs_print_message ("%s", _("undelfs: reading block bitmap..."));
     if (ext2fs_read_block_bitmap (fs))
     {
         message (D_ERROR, undelfserr, _("Cannot load block bitmap from:\n%s"), ext2_fname);
@@ -395,29 +392,31 @@ undelfs_opendir (const vfs_path_t * vpath)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void *
+static struct vfs_dirent *
 undelfs_readdir (void *vfs_info)
 {
-    static union vfs_dirent undelfs_readdir_data;
-    static char *const dirent_dest = undelfs_readdir_data.dent.d_name;
+    struct vfs_dirent *dirent;
 
     if (vfs_info != fs)
     {
-        message (D_ERROR, undelfserr, _("vfs_info is not fs!"));
+        message (D_ERROR, undelfserr, "%s", _("vfs_info is not fs!"));
         return NULL;
     }
     if (readdir_ptr == num_delarray)
         return NULL;
     if (readdir_ptr < 0)
-        strcpy (dirent_dest, readdir_ptr == -2 ? "." : "..");
+        dirent = vfs_dirent_init (NULL, readdir_ptr == -2 ? "." : "..", 0);     /* FIXME: inode */
     else
+    {
+        char dirent_dest[MC_MAXPATHLEN];
+
         g_snprintf (dirent_dest, MC_MAXPATHLEN, "%ld:%d",
                     (long) delarray[readdir_ptr].ino, delarray[readdir_ptr].num_blocks);
+        dirent = vfs_dirent_init (NULL, dirent_dest, 0);        /* FIXME: inode */
+    }
     readdir_ptr++;
 
-    compute_namelen (&undelfs_readdir_data.dent);
-
-    return &undelfs_readdir_data;
+    return dirent;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -451,7 +450,7 @@ undelfs_open (const vfs_path_t * vpath, int flags, mode_t mode)
 
     if (!ext2_fname || strcmp (ext2_fname, file))
     {
-        message (D_ERROR, undelfserr, _("You have to chdir to extract files first"));
+        message (D_ERROR, undelfserr, "%s", _("You have to chdir to extract files first"));
         g_free (file);
         g_free (f);
         return 0;
@@ -597,7 +596,7 @@ undelfs_read (void *vfs_info, char *buffer, size_t count)
     retval = ext2fs_block_iterate (fs, p->inode, 0, NULL, undelfs_dump_read, p);
     if (retval)
     {
-        message (D_ERROR, undelfserr, _("while iterating over blocks"));
+        message (D_ERROR, undelfserr, "%s", _("while iterating over blocks"));
         return -1;
     }
     if (p->error_code && !p->finished)
@@ -637,6 +636,9 @@ undelfs_stat_int (int inode_index, struct stat *buf)
     buf->st_atime = delarray[inode_index].dtime;
     buf->st_ctime = delarray[inode_index].dtime;
     buf->st_mtime = delarray[inode_index].dtime;
+#ifdef HAVE_STRUCT_STAT_ST_MTIM
+    buf->st_atim.tv_nsec = buf->st_mtim.tv_nsec = buf->st_ctim.tv_nsec = 0;
+#endif
     return 0;
 }
 
@@ -670,9 +672,9 @@ undelfs_lstat (const vfs_path_t * vpath, struct stat *buf)
 
     if (!ext2_fname || strcmp (ext2_fname, file))
     {
-        message (D_ERROR, undelfserr, _("You have to chdir to extract files first"));
         g_free (file);
         g_free (f);
+        message (D_ERROR, undelfserr, "%s", _("You have to chdir to extract files first"));
         return 0;
     }
     inode_index = undelfs_getindex (f);
@@ -759,12 +761,12 @@ undelfs_getid (const vfs_path_t * vpath)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 undelfs_nothingisopen (vfsid id)
 {
     (void) id;
 
-    return !undelfs_usage;
+    return (undelfs_usage == 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -817,26 +819,28 @@ com_err (const char *whoami, long err_code, const char *fmt, ...)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-init_undelfs (void)
+vfs_init_undelfs (void)
 {
-    vfs_undelfs_ops.name = "undelfs";
-    vfs_undelfs_ops.prefix = "undel";
-    vfs_undelfs_ops.init = undelfs_init;
-    vfs_undelfs_ops.open = undelfs_open;
-    vfs_undelfs_ops.close = undelfs_close;
-    vfs_undelfs_ops.read = undelfs_read;
-    vfs_undelfs_ops.opendir = undelfs_opendir;
-    vfs_undelfs_ops.readdir = undelfs_readdir;
-    vfs_undelfs_ops.closedir = undelfs_closedir;
-    vfs_undelfs_ops.stat = undelfs_stat;
-    vfs_undelfs_ops.lstat = undelfs_lstat;
-    vfs_undelfs_ops.fstat = undelfs_fstat;
-    vfs_undelfs_ops.chdir = undelfs_chdir;
-    vfs_undelfs_ops.lseek = undelfs_lseek;
-    vfs_undelfs_ops.getid = undelfs_getid;
-    vfs_undelfs_ops.nothingisopen = undelfs_nothingisopen;
-    vfs_undelfs_ops.free = undelfs_free;
-    vfs_register_class (&vfs_undelfs_ops);
+    /* NULLize vfs_s_subclass members */
+    memset (&undelfs_subclass, 0, sizeof (undelfs_subclass));
+
+    vfs_init_class (vfs_undelfs_ops, "undelfs", VFSF_UNKNOWN, "undel");
+    vfs_undelfs_ops->init = undelfs_init;
+    vfs_undelfs_ops->open = undelfs_open;
+    vfs_undelfs_ops->close = undelfs_close;
+    vfs_undelfs_ops->read = undelfs_read;
+    vfs_undelfs_ops->opendir = undelfs_opendir;
+    vfs_undelfs_ops->readdir = undelfs_readdir;
+    vfs_undelfs_ops->closedir = undelfs_closedir;
+    vfs_undelfs_ops->stat = undelfs_stat;
+    vfs_undelfs_ops->lstat = undelfs_lstat;
+    vfs_undelfs_ops->fstat = undelfs_fstat;
+    vfs_undelfs_ops->chdir = undelfs_chdir;
+    vfs_undelfs_ops->lseek = undelfs_lseek;
+    vfs_undelfs_ops->getid = undelfs_getid;
+    vfs_undelfs_ops->nothingisopen = undelfs_nothingisopen;
+    vfs_undelfs_ops->free = undelfs_free;
+    vfs_register_class (vfs_undelfs_ops);
 }
 
 /* --------------------------------------------------------------------------------------------- */
