@@ -1,7 +1,7 @@
 /*
    lib/vfs - vfs_path_t serialize/deserialize functions
 
-   Copyright (C) 2011-2019
+   Copyright (C) 2011-2021
    Free Software Foundation, Inc.
 
    Written by:
@@ -51,13 +51,13 @@ setup (void)
     vfs_init_localfs ();
     vfs_setup_work_dir ();
 
-    vfs_init_class (&vfs_test_ops1, "testfs1", VFS_NOLINKS | VFS_REMOTE, "test1");
+    vfs_init_class (&vfs_test_ops1, "testfs1", VFSF_NOLINKS | VFSF_REMOTE, "test1");
     vfs_register_class (&vfs_test_ops1);
 
-    vfs_init_class (&vfs_test_ops2, "testfs2", VFS_UNKNOWN, "test2");
+    vfs_init_class (&vfs_test_ops2, "testfs2", VFSF_UNKNOWN, "test2");
     vfs_register_class (&vfs_test_ops2);
 
-    vfs_init_class (&vfs_test_ops3, "testfs3", VFS_UNKNOWN, "test3");
+    vfs_init_class (&vfs_test_ops3, "testfs3", VFSF_UNKNOWN, "test3");
     vfs_register_class (&vfs_test_ops3);
 
     mc_global.sysconfig_dir = (char *) TEST_SHARE_DIR;
@@ -144,7 +144,7 @@ START_TEST (test_path_serialize)
     vpath = vfs_path_from_str_flags (ETALON_PATH_STR, VPF_USE_DEPRECATED_PARSER);
     serialized_vpath = vfs_path_serialize (vpath, &error);
 
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 
     if (error != NULL)
         g_error_free (error);
@@ -174,7 +174,7 @@ START_TEST (test_path_deserialize)
     mctest_assert_ptr_ne (vpath, NULL);
     mctest_assert_str_eq (vfs_path_as_str (vpath), ETALON_PATH_URL_STR);
 
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 
 }
 /* *INDENT-OFF* */
@@ -186,11 +186,9 @@ END_TEST
 int
 main (void)
 {
-    int number_failed;
+    TCase *tc_core;
 
-    Suite *s = suite_create (TEST_SUITE_NAME);
-    TCase *tc_core = tcase_create ("Core");
-    SRunner *sr;
+    tc_core = tcase_create ("Core");
 
     tcase_add_checked_fixture (tc_core, setup, teardown);
 
@@ -199,13 +197,7 @@ main (void)
     tcase_add_test (tc_core, test_path_deserialize);
     /* *********************************** */
 
-    suite_add_tcase (s, tc_core);
-    sr = srunner_create (s);
-    srunner_set_log (sr, "path_serialize.log");
-    srunner_run_all (sr, CK_ENV);
-    number_failed = srunner_ntests_failed (sr);
-    srunner_free (sr);
-    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+    return mctest_run_all (tc_core);
 }
 
 /* --------------------------------------------------------------------------------------------- */
