@@ -1,6 +1,6 @@
 /* lib/vfs - test vfs_path_from_str_flags() function
 
-   Copyright (C) 2013-2019
+   Copyright (C) 2013-2021
    Free Software Foundation, Inc.
 
    Written by:
@@ -98,7 +98,7 @@ START_PARAMETRIZED_TEST (test_strip_home, test_strip_home_ds)
     /* then */
     mctest_assert_str_eq (actual_result->str, data->expected_result);
 
-    vfs_path_free (actual_result);
+    vfs_path_free (actual_result, TRUE);
 }
 /* *INDENT-OFF* */
 END_PARAMETRIZED_TEST
@@ -109,11 +109,9 @@ END_PARAMETRIZED_TEST
 int
 main (void)
 {
-    int number_failed;
+    TCase *tc_core;
 
-    Suite *s = suite_create (TEST_SUITE_NAME);
-    TCase *tc_core = tcase_create ("Core");
-    SRunner *sr;
+    tc_core = tcase_create ("Core");
 
     tcase_add_checked_fixture (tc_core, setup, teardown);
 
@@ -121,13 +119,7 @@ main (void)
     mctest_add_parameterized_test (tc_core, test_strip_home, test_strip_home_ds);
     /* *********************************** */
 
-    suite_add_tcase (s, tc_core);
-    sr = srunner_create (s);
-    srunner_set_log (sr, "vfs_path_from_str_flags.log");
-    srunner_run_all (sr, CK_ENV);
-    number_failed = srunner_ntests_failed (sr);
-    srunner_free (sr);
-    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+    return mctest_run_all (tc_core);
 }
 
 /* --------------------------------------------------------------------------------------------- */
